{ *************************************************************************** }
{                                                                             }
{ Hotzonu 2 Memo Window Unit                                                  }
{                                                                             }
{ 2001-04  Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Memo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ToolWin, ExtCtrls, IniFiles,
  ZonuClass, Hzbconst, ZonuConf, Menus, ActnList;

type
  TMemoData = class;

  TZonuMemoWindow = class(TForm)
    StatusBar1: TStatusBar;
    TabControl1: TTabControl;
    Panel1: TPanel;
    PopupMenu1: TPopupMenu;
    A1: TMenuItem;
    N1: TMenuItem;
    Rename1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    Memo1: TMemo;
    ActionList1: TActionList;
    actSelectAll: TAction;
    procedure FormCreate(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TabControl1Changing(Sender: TObject;
      var AllowChange: Boolean);
    procedure FormDeactivate(Sender: TObject);
    procedure A1Click(Sender: TObject);
    procedure Rename1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure N4Click(Sender: TObject);
    procedure TabControl1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure actSelectAllExecute(Sender: TObject);
  private
    { Private 錾 }
    FSaveHeight: integer;
  public
    { Public 錾 }
  end;

  TMemoData = class(TObject)
  private
    FTopIndex: integer;
    FID: string;
    FTitle: string;
    FLines: TStrings;
    FSelStart: integer;
    FSelLength: integer;
    procedure SetLines(const Value: TStrings);
  public
    constructor Create;
    destructor  Destroy; override;
    procedure SaveToFile;
    procedure DeleteFile;
    property  ID: string    read  FID write FID;
    property  Title: string read  FTitle  write FTitle;
    property  TopIndex: integer read  FTopIndex write FTopIndex;
    property  SelStart: integer read  FSelStart write FSelStart;
    property  SelLength: integer read  FSelLength write FSelLength;
    property  Lines: TStrings read  FLines  write SetLines;
  end;

var
  ZonuMemoWindow: TZonuMemoWindow;

implementation

{$R *.dfm}

{ TMemoData }

constructor TMemoData.Create;
begin
  FLines  :=  TStringList.Create;
end;

procedure TMemoData.DeleteFile;
var
  s: string;
begin
  s :=  ZonuConfig.UsersPath + PATH_MEMO + ID + '.txt';
  if  (FileExists(s))  then
    Windows.DeleteFile(PChar(s));
end;

destructor TMemoData.Destroy;
begin
  FLines.Free;
  inherited;
end;

procedure TMemoData.SaveToFile;
begin
  Lines.SaveToFile(ZonuConfig.UsersPath + PATH_MEMO + ID + '.txt');
end;

procedure TMemoData.SetLines(const Value: TStrings);
begin
  FLines.Assign(Value);
end;

procedure TZonuMemoWindow.FormCreate(Sender: TObject);
var
  MemoData  : TMemoData;
  ini : TMemIniFile;
  s   : string;
  i   : integer;
begin
  if  (not DirectoryExists(ZonuConfig.UsersPath + PATH_MEMO)) then  begin
    ForceDirectories(ZonuConfig.UsersPath + PATH_MEMO);
  end;
  s   :=  ZonuConfig.UsersPath + PATH_MEMO + FILE_MEMO_INI;
  if  (not FileExists(s)) then  begin
    CopyFile(PChar(ZonuConfig.AppPath + PATH_DEF + PATH_MEMO + FILE_MEMO_INI),
             PChar(s),
             True);
    CopyFile(PChar(ZonuConfig.AppPath + PATH_DEF + PATH_MEMO + 'mem00001.txt'),
             PChar(ZonuConfig.UsersPath + PATH_MEMO + 'mem00001.txt'),
             True);
  end;
  ini :=  TMemIniFile.Create(s);
  if  (ini.ReadInteger('MEMO', 'COUNT', 0) = 0) then  begin
    ini.WriteInteger('LIST', 'USED', 1);
    ini.WriteInteger('MEMO', 'COUNT', 1);
    ini.WriteString('MEMO',  'MEMO1', '̂P');
    ini.WriteString('MEMO',  'ID1', 'mem00001');
    ini.WriteInteger('MEMO', 'TOPINDEX1', 0);
    ini.UpdateFile;
  end;
  s :=  ZonuConfig.ZonuMemoFontName;
  if  (s <> '') then  Memo1.Font.Name :=  s;
  Memo1.Font.Size :=  ZonuConfig.ZonuMemoFontSize;
  TabControl1.Tabs.Clear;
  for i :=  0 to  ini.ReadInteger('MEMO', 'COUNT', 0) - 1 do  begin
    MemoData  :=  TMemoData.Create;
    MemoData.ID       :=  ini.ReadString('MEMO', 'ID' + IntToStr(i + 1), '');
    MemoData.Title    :=  ini.ReadString('MEMO', 'MEMO' + IntToStr(i + 1), '');
    MemoData.TopIndex :=  ini.ReadInteger('MEMO', 'TOPINDEX' + IntToStr(i + 1), 0);
    MemoData.SelStart :=  ini.ReadInteger('MEMO', 'SELSTART' + IntToStr(i + 1), 0);
    MemoData.SelLength:=  ini.ReadInteger('MEMO', 'SELLEN' + IntToStr(i + 1), 0);
    MemoData.Lines.LoadFromFile(ZonuConfig.UsersPath + PATH_MEMO +
                                MemoData.ID + '.txt');
    TabControl1.Tabs.Objects[TabControl1.Tabs.Add(MemoData.Title)]  :=  MemoData;
  end;
  TabControl1.OnChange(TabControl1);
  FSaveHeight :=  Height;
end;

procedure TZonuMemoWindow.FormActivate(Sender: TObject);
begin
  SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE);
  AlphaBlend  :=  False;
  Height  :=  FSaveHeight;
  Memo1.SetFocus;
end;

procedure TZonuMemoWindow.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE or SWP_NOMOVE);
end;

procedure TZonuMemoWindow.TabControl1Change(Sender: TObject);
var
  MemoData: TMemoData;
begin
  MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
  if  (MemoData <> nil) then  begin
    Memo1.Lines.Assign(MemoData.Lines);
    Memo1.SelStart  :=  MemoData.SelStart;
    Memo1.SelLength :=  MemoData.SelLength;
    Memo1.Perform(EM_LINESCROLL, 0, MemoData.TopIndex);
  end;
end;

procedure TZonuMemoWindow.TabControl1Changing(Sender: TObject;
  var AllowChange: Boolean);
var
  MemoData: TMemoData;
begin
  MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
  if  (MemoData <> nil) then  begin
    MemoData.Lines.Assign(Memo1.Lines);
    MemoData.SelStart     :=  Memo1.SelStart;
    MemoData.SelLength    :=  Memo1.SelLength;
    MemoData.TopIndex :=  Memo1.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
    MemoData.SaveToFile;
  end;
  AllowChange :=  True;
end;

procedure TZonuMemoWindow.FormDeactivate(Sender: TObject);
var
  MemoData: TMemoData;
begin
  if  (TabControl1.TabIndex > -1) then  begin
    MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
    if  (MemoData <> nil) then  begin
      MemoData.Lines.Assign(Memo1.Lines);
      MemoData.SelStart     :=  Memo1.SelStart;
      MemoData.SelLength    :=  Memo1.SelLength;
      MemoData.TopIndex :=  Memo1.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
      MemoData.SaveToFile;
    end;
  end;
  if  (ZonuConfig.ZonuMemoTrans)  then
  begin
    //TransWindow(True);
    AlphaBlend  :=  True;
    AlphaBlendValue :=  128;
  end;
  FSaveHeight :=  Height;
  if  (ZonuConfig.ZonuMemoExpand) then  begin
    {
    if  (TabControl1.Tabs.Count > 0)  then  begin
      r :=  TabControl1.TabRect(0);
      Height      :=  r.Bottom - r.Top + (Self.ClientHeight - Self.Height);
    end else begin
      Height      :=  0;
    end;
    }
    Height      :=  0;
  end;
end;

procedure TZonuMemoWindow.A1Click(Sender: TObject);
var
  MemoData  : TMemoData;
  ini : TMemIniFile;
  s, ID : string;
  n : integer;
begin
  s :=  'VKy[W';
  if  (InputQuery('y[W̒ǉ', 'O', s)) then  begin
    ini :=  TMemIniFile.Create(ZonuConfig.UsersPath + PATH_MEMO + FILE_MEMO_INI);
    n   :=  ini.ReadInteger('LIST', 'USED', 0);
    ID  :=  Format('mem%.5d', [n + 1]);
    ini.WriteInteger('LIST', 'USED', n + 1);
    ini.UpdateFile;
    ini.Free;
    MemoData        :=  TMemoData.Create;
    MemoData.ID     :=  ID;
    MemoData.Title  :=  s;
    MemoData.TopIndex   :=  0;
    MemoData.SelStart   :=  0;
    MemoData.SelLength  :=  0;
    MemoData.Lines.Clear;
    n :=  TabControl1.Tabs.Add(s);
    TabControl1.Tabs.Objects[n] :=  MemoData;
    //TabControl1.TabIndex  :=  n;
  end;
end;

procedure TZonuMemoWindow.Rename1Click(Sender: TObject);
var
  MemoData  : TMemoData;
  s : string;
begin
  MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
  s :=  MemoData.Title;
  if  (InputQuery('O̕ύX', 'O', s)) then  begin
    MemoData.Title      :=  s;
  end;
end;

procedure TZonuMemoWindow.N1Click(Sender: TObject);
var
  MemoData  : TMemoData;
begin
  MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
  if  (MessageDlg('y[W폜܂H', mtWarning, [mbYes, mbNo], 0) = mrYes)
  then  begin
    MemoData.DeleteFile;
    MemoData.Free;
    TabControl1.Tabs.Delete(TabControl1.TabIndex);
  end;
  if  (TabControl1.Tabs.Count > 0)  then  begin
    TabControl1.TabIndex  :=  0;
    TabControl1.OnChange(Self);
  end else begin
    Memo1.Lines.Clear;
  end;
end;

procedure TZonuMemoWindow.FormDestroy(Sender: TObject);
var
  MemoData: TMemoData;
  ini : TMemIniFile;
  i : integer;
begin
  if  (TabControl1.Tabs.Count > 0)  then  begin
    MemoData  :=  TMemoData(TabControl1.Tabs.Objects[TabControl1.TabIndex]);
    if  (MemoData <> nil) then  begin
      MemoData.Lines.Assign(Memo1.Lines);
      MemoData.SelStart     :=  Memo1.SelStart;
      MemoData.SelLength    :=  Memo1.SelLength;
      MemoData.TopIndex     :=  Memo1.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
      MemoData.SaveToFile;
    end;
  end;
  ini :=  TMemIniFile.Create(ZonuConfig.UsersPath + PATH_MEMO + FILE_MEMO_INI);
  ini.EraseSection('MEMO');
  ini.WriteInteger('MEMO', 'COUNT', TabControl1.Tabs.Count);
  for i :=  0 to  TabControl1.Tabs.Count - 1  do  begin
    MemoData  :=  TMemoData(TabControl1.Tabs.Objects[i]);
    ini.WriteString('MEMO', 'MEMO' + IntToStr(i + 1), MemoData.Title);
    ini.WriteString('MEMO', 'ID' + IntToStr(i + 1), MemoData.ID);
    ini.WriteInteger('MEMO', 'TOPINDEX' + IntToStr(i + 1), MemoData.TopIndex);
    ini.WriteInteger('MEMO', 'SELSTART' + IntToStr(i + 1), MemoData.SelStart);
    ini.WriteInteger('MEMO', 'SELLEN' + IntToStr(i + 1), MemoData.SelLength);
  end;
  ini.UpdateFile;
  ini.Free;
end;

procedure TZonuMemoWindow.FormShow(Sender: TObject);
begin
  SetBounds(
    ZonuConfig.ZonuMemoPosLeft,
    ZonuConfig.ZonuMemoPosTop,
    ZonuConfig.ZonuMemoWidth,
    ZonuConfig.ZonuMemoHeight);
  TabControl1.MultiLine :=  ZonuConfig.ZonuMemoMultiline;
  N4.Checked            :=  ZonuConfig.ZonuMemoMultiline;
end;

procedure TZonuMemoWindow.FormHide(Sender: TObject);
begin
  ZonuConfig.ZonuMemoPosTop     :=  Top;
  ZonuConfig.ZonuMemoPosLeft    :=  Left;
  ZonuConfig.ZonuMemoWidth      :=  Width;
  ZonuConfig.ZonuMemoHeight     :=  Height;
  ZonuConfig.ZonuMemoMultiline  :=  TabControl1.MultiLine;
end;

procedure TZonuMemoWindow.N4Click(Sender: TObject);
begin
  N4.Checked  :=  not N4.Checked;
  TabControl1.MultiLine :=  N4.Checked;
end;

procedure TZonuMemoWindow.TabControl1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if  (Button = mbRight)  then  begin
    PopupMenu1.Popup(TabControl1.ClientOrigin.X + X,
                     TabControl1.ClientOrigin.Y + Y);
  end;
end;

procedure TZonuMemoWindow.actSelectAllExecute(Sender: TObject);
begin
  Memo1.SelectAll;
end;

end.
