{ *************************************************************************** }
{                                                                             }
{ Hotzonu JBBS Script Unit                                                    }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuJbbs;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst;

type
  PParamData  = ^TParamData;
  TParamData  = record
    Subject : string;
  end;

  TJbbsScript = class(TBBS)
  private
    { private }
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, Async, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, Async, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
  end;

implementation

{ TJbbsScript }

//
//  Xbhꗗ̎擾@
//
function TJbbsScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function TJbbsScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmCGI;
end;

//
//  http://ւ̃Nǉ
//
function TJbbsScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function TJbbsScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function TJbbsScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure TJbbsScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, Async, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', Async);
end;

//
//  Xbhꗗ擾
//
procedure TJbbsScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  if  (Data <> '')  then  begin
    dl  :=  TThreadListDownloader(Downloader);
    ParseSubject(Data, AFileName, ATitle, ACount);
    ThreadItem                    :=  TThreadItem.Create;
    ThreadItem.BBS                :=  Self;
    ThreadItem.Site               :=  dl.Site;
    ThreadItem.BoardID            :=  dl.Board.BoardID;
    ThreadItem.No                 :=  0;
    ThreadItem.DatFileName        :=  AFileName;
    ThreadItem.Title              :=  ATitle;
    ThreadItem.AgeCount           :=  ACount;
    ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
    ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName;
    dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
  end;
end;

//
//  Xbhꗗ擾I
//
procedure TJbbsScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure TJbbsScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure TJbbsScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  Parse2chSubject(subject, FileName, Title, Count);
  FileName  :=  ChangeFileExt(FileName, '.dat');
end;

//
//  X̉
//
procedure TJbbsScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;
  ParseJbbsDat(ALine, Index, ARes, ASubject);
end;

//
//  \URL
//
function TJbbsScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  GetReadCgiUrl(ThreadItem) + '&LAST=100';
end;

//
//  Xbh̃_E[h
//
procedure TJbbsScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, Async, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Subject         :=  '';
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Downloader.BBS.GetReadCgiUrl(Downloader.ThreadItem);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetCgiDiff(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), Async);
end;

//
//  X擾
//
procedure TJbbsScript.ZonuDataRes(Sender: TObject; Downloader: TDownloader;
  Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean);
var
  dl  : TResDownloader;
  ParamData:  PParamData;
  ASubject :  string;
  Res : TRes;
  s   : string;
  b   : boolean;
begin
  dl  :=  TResDownloader(Downloader);
  ParamData :=  PParamData(Downloader.ParamData);
  b         :=  False;
  if  ((AnsiPos('<dt>', Data) = 1) and (AnsiPos('<dd>', Data) > 0)) then  begin
    s :=  Data;
    b :=  True;
  end;
  if  (not b) then  begin
    if  ((AnsiPos('<dt>', Data) = 1) and (AnsiPos('<dd>', Data) = 0)) then  begin
      dl.Temp :=  Data;
    end else
    if  ((AnsiPos('<dt>', Data) = 0) and (AnsiPos('<dd>', Data) > 0)) then  begin
      s :=  dl.Temp +  Data;
      b :=  True;
    end;
  end;
  if  (b) then  begin
    Res :=  TRes.Create;
    ParseDat(S, 0, Res, ASubject);
    Res.BBS         :=  dl.BBS;
    Res.Site        :=  dl.Site;
    Res.ThreadTitle :=  ASubject;
    if  (dl.ThreadItem.Title = '')  then  begin
      if  (ParamData.Subject <> '')  then  begin
        dl.ThreadItem.Title :=  ParamData.Subject;
      end;
    end;
    dl.FireDataEvt(Res, DownloadSize, TotalSize);
  end else
  if  (AnsiPos('<title>', Data) = 1) then  begin
    s :=  Copy(Data, 8, Length(Data));
    ParamData.Subject  :=  Copy(s, 1, AnsiPos('</title>', s) - 1);
  end;
end;

//
//  X擾I
//
procedure TJbbsScript.ZonuEndRes(Sender: TObject; Downloader: TDownloader;
  Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure TJbbsScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg: string);
begin
  inherited;

end;

//
//  
//
procedure TJbbsScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

//
//  VX
//
procedure TJbbsScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

function TJbbsScript.GetRelativeLink: boolean;
begin
  Result  :=  False;
end;

end.
