{ *************************************************************************** }
{                                                                             }
{ Hotzonu RSS Unit                                                            }
{                                                                             }
{ 2001-05 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit ZonuRSS;

interface

uses
  Classes, SysUtils, IniFiles, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst,
  ZonuCustom;

type
  TRssScript = class(TBBS)
  private
    { private }
    FParseOk: boolean;
    FMoreData: boolean;
    FData: string;
    pubDate: string;
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    function GetWebRedirect: boolean; override;
  public
    { public }
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, Async, InfSave: boolean); override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, Async, InfSave, AutoPilot: boolean); override;
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); overload; override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); overload; override;  
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
  end;

implementation

{ TRssScript }

uses
  MonaUtils, Zonutils, HttpProt;



//
//  Xbhꗗ_E[h
//
procedure TRssScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, Async, InfSave: boolean);
var
  //ParamData : PParamData;
  FZonuHttp : TZonuHttp;
  //FGikoBas  : TGikoBas;
begin
   FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  'http://' + (Downloader.Board.BoardPath) +
                                '/' +
                                StringReplace(Downloader.Board.RemoteBoardID, '@', '/', [rfReplaceAll]);
                                // rss.xml
                                // index.rdf
  //18 begn
  //FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.NoCache         :=  True; //RSS͏ɃLbVǂ܂Ȃ
  //18 end
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', Async);
  FData :=  '';
end;

//
//  Xbhꗗ擾
//
procedure TRssScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  dl        :=  TThreadListDownloader(Downloader);
  FMoreData :=  False;
  FData     :=  FData + Data;
  repeat
    FParseOk  :=  False;
    ParseSubject(FData, AFileName, ATitle, ACount);
    if  (FParseOk)  then  begin
      ThreadItem                    :=  TThreadItem.Create;
      ThreadItem.BBS                :=  Self;
      ThreadItem.Site               :=  dl.Site;
      ThreadItem.BoardID            :=  dl.Board.BoardID;
      ThreadItem.No                 :=  0;
      ThreadItem.DatFileName        :=  UrlToFileName(AFileName);
      ThreadItem.Title              :=  ATitle;
      ThreadItem.AgeCount           :=  ACount;
      ThreadItem.RemoteDatFileName  :=  AFileName; //GetDatUrl(dl.Board, AFileName);
      ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + UrlToFileName(AFileName);
      ThreadItem.DatSince           :=  pubDate;
      dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
    end;
  until (not FMoreData);
end;

procedure TRssScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
var
  s, sx, Data: string;
  n,i : integer;
begin
  inherited;
  Data  :=  Subject;
  n     :=  AnsiPos('<item>', Data);
  if  (n = 0) then  begin
    n     :=  AnsiPos('<item ', Data);
  end;
  i     :=  AnsiPos('</item>', Data);
  if  (n > 0) and (i > 0) then  begin
    Data  :=  Copy(Data, n + 6, Length(Data));
    sx    :=  Copy(Data, 1, AnsiPos('</item>', Data) - 1);
    Data  :=  Copy(Data, AnsiPos('</item>', Data) + 7, Length(Data));

    s   :=  Copy(sx, AnsiPos('<title>', sx) + 7, Length(sx));
    s   :=  Copy(s, 1, AnsiPos('</title>', s) - 1);
    Title :=  s;

    s   :=  Copy(sx, AnsiPos('<link>', sx) + 6, Length(sx));
    s   :=  Copy(s, 1, AnsiPos('</link>', s) - 1);
    FileName :=  s;

    s   :=  Copy(sx, AnsiPos('<pubDate>', sx) + 9, Length(sx));
    s   :=  Copy(s, 1, AnsiPos('</pubDate>', s) - 1);
    pubDate :=  s;

    s   :=  Copy(sx, AnsiPos('<dc:date>', sx) + 9, Length(sx));
    s   :=  Copy(s, 1, AnsiPos('</dc:date>', s) - 1);
    pubDate :=  s;

    Count :=  1;

    FParseOk  :=  (FileName <> '');

    FMoreData   :=  True;
  end else begin
    FMoreData   :=  False;
  end;
  FData :=  Data;
end;

function TRssScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  ThreadItem.DatFileName;
end;

procedure TRssScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, Async, InfSave, AutoPilot: boolean);
var
  ThreadItem: TThreadItem;
  inf: TMemIniFile;
  s: string;
begin
  ThreadItem := TResDownloader(Downloader).ThreadItem;
  if  (ZonuEnviron.Cached)  and (not Downloader.NotModified) then  begin
    s :=  ExtractFilePath(ThreadItem.LocalDatFileName);
    if  (not DirectoryExists(s)) and (s <> '')  then  begin
      ForceDirectories(s);
    end;
    //
    if  (s <> '') then  begin
      ThreadItem.LastLoadDate :=  FormatDateTime('yyyy/mm/dd hh:nn:ss', Now);
      inf :=  TMemIniFile.Create(ChangeFileExt(ThreadItem.LocalDatFileName, '.inf'));
      inf.WriteInteger('LOG', 'COUNT', ThreadItem.DownloadCount);
      inf.WriteString('LOG', 'TITLE', ThreadItem.Title);
      inf.WriteString('LOG', 'BOARDID', ThreadItem.BoardID);
      inf.WriteString('LOG', 'LAST_LOAD', ThreadItem.LastLoadDate);
      inf.UpdateFile;
      inf.Free;
    end;
  end;
  Downloader.FireRedirect(ThreadItem.RemoteDatFileName);
  Downloader.FireEndDownload(False);
end;

function TRssScript.GetWebRedirect: boolean;
begin
  Result  :=  True;
end;

function TRssScript.GetNeedHRefLink: boolean;
begin
  Result  :=  False;
end;

function TRssScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

function TRssScript.GetRelativeLink: boolean;
begin
  Result  :=  False;
end;

function TRssScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmText;  
end;

function TRssScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

function TRssScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

procedure TRssScript.ParseDat(const ALine: string; const index: integer;
  var ARes: TRes; out ASubject: string);
begin
  inherited;

end;

procedure TRssScript.SendRes(Uploader: TResUploader);
begin
  inherited;

end;

procedure TRssScript.SendThread(Uploader: TThreadUploader);
begin
  inherited;

end;

end.



